package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.SlotAttribute;
import org.bouml.plugout.apibase.SlotRelation;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;

public class UmlClassInstance extends UmlItem {
	/**
	 * Returns a new class instance
	 * In case it cannot be created ('parent' cannot contain it etc ...) return
	 * 0 in C++ and produce a PlugoutException in Java
	 */
	public static UmlClassInstance create(UmlItem parent, String name, UmlClass type) throws PlugoutException {
		UmlCom.send_cmd(parent.identifier_(), OnInstanceCmd.createCmd, anItemKind.aClassInstance, type.identifier_());
		final UmlClassInstance result = (UmlClassInstance)UmlItem.read_();
		if(result != null) {
			parent.reread_children_if_needed_();
			if(name != null) {
				result.set_Name(name);
			}
		} else {
			throw new PlugoutException("Cannot create the class instance");
		}
		return result;
	}

	private UmlObjectDiagram _assoc_diagram;

	private UmlClass _type;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlClassInstance(long id, String s) {
		super(id, s);
	}

	/**
	 * Add the slot (does nothing if it already exist)
	 * On error : return FALSE in C++, produce a PlugoutException in Java
	 */
	public void add_Relation(UmlRelation relation, UmlClassInstance other) throws PlugoutException {
		final UmlItem[] v = new UmlItem[2];
		v[0] = relation;
		v[1] = other;
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.addRelationCmd, v);
		UmlCom.check();
	}

	/**
	 * returns the optional associated diagram
	 */
	public UmlObjectDiagram associatedDiagram() {
		read_if_needed_();
		return _assoc_diagram;
	}

	/**
	 * Returns the attributes having a value
	 */
	public SlotAttribute[] attributesValue() {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.attributesCmd, (byte)0);
		final int n = UmlCom.read_unsigned();
		final SlotAttribute[] v = new SlotAttribute[n];
		for(int index = 0 ; index != n ; index += 1) {
			final UmlAttribute at = (UmlAttribute)UmlItem.read_();
			v[index] = new SlotAttribute(at, UmlCom.read_string());
		}
		return v;
	}

	/**
	 * Returns all the attributes of the class instance, including the inherited
	 */
	public UmlAttribute[] availableAttributes() {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.attributesCmd, (byte)1);
		final int n = UmlCom.read_unsigned();
		final UmlAttribute[] v = new UmlAttribute[n];
		for(int index = 0 ; index != n ; index += 1) {
			v[index] = (UmlAttribute)UmlItem.read_();
		}
		return v;
	}

	/**
	 * Returns all the possible relations from the current instance to 'other',
	 * including the inherited
	 */
	public UmlRelation[] availableRelations(UmlClassInstance other) {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.relationsCmd, other.identifier_());
		final int n = UmlCom.read_unsigned();
		final UmlRelation[] v = new UmlRelation[n];
		for(int index = 0 ; index != n ; index += 1) {
			v[index] = (UmlRelation)UmlItem.read_();
		}
		return v;
	}

	@Override
	public anItemKind kind() {
		return anItemKind.aClassInstance;
	}

	@Override
	protected void read_uml_() {
		_assoc_diagram = (UmlObjectDiagram)UmlItem.read_();
		super.read_uml_();
		_type = (UmlClass)UmlItem.read_();
	}

	/**
	 * Returns the attributes having a value
	 */
	public SlotRelation[] relationsValue() {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.relationsCmd, (long)0);
		final int n = UmlCom.read_unsigned();
		final SlotRelation[] v = new SlotRelation[n];
		for(int index = 0 ; index != n ; index += 1) {
			final UmlRelation rel = (UmlRelation)UmlItem.read_();
			v[index] = new SlotRelation(rel, (UmlClassInstance)UmlItem.read_());
		}
		return v;
	}

	/**
	 * Remove the slot (does nothing if it doesn't exist)
	 * On error : return FALSE in C++, produce a PlugoutException in Java
	 */
	public void remove_Relation(UmlRelation relation, UmlClassInstance other) throws PlugoutException {
		final UmlItem[] v = new UmlItem[2];
		v[0] = relation;
		v[1] = other;
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.removeRelationCmd, v);
		UmlCom.check();
	}

	/**
	 * sets the associated diagram, arg may be null to unset it
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_AssociatedDiagram(UmlObjectDiagram d) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setAssocDiagramCmd, d == null ? (long)0 : d.identifier_());
		UmlCom.check();
		_assoc_diagram = d;
	}

	/**
	 * Remove the slot if the value is null. Else set the value for the given
	 * attribute, replacing it if the slot already exist. On error : return
	 * FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_AttributeValue(UmlAttribute attribute, String value) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setAttributeCmd, attribute.identifier_(), value);
		UmlCom.check();
	}

	/**
	 * set the type
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Type(UmlClass v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setTypeCmd, v.identifier_());
		UmlCom.check();
		_type = v;
	}

	/**
	 * return the type
	 */
	public UmlClass type() {
		read_if_needed_();
		return _type;
	}
}
