package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.aVisibility;

/**
 * This class manages 'true' class's items : attributes, relation, operation and
 * sub-classes
 */
public abstract class UmlClassMember extends UmlClassItem {
	private boolean _class_member;

	private String _constraint;

	private aVisibility _cpp_visibility;

	private String _java_annotation;

	private aVisibility _visibility;

	private boolean _volatile;

	protected UmlClassMember(long id, String n) {
		super(id, n);
	}

	/**
	 * return the constraint
	 */
	public String constraint() {
		read_if_needed_();
		return _constraint;
	}

	/**
	 * Special case for C++, thank to the 'friend class X' it may be usefull to
	 * have a C++ visibility != other visibility In case the C++ visibility must
	 * follow the other, return DefaultVisibility
	 */
	public aVisibility cppVisibility() {
		read_if_needed_();
		return _cpp_visibility;
	}

	/**
	 * Indicate if the member is a 'class member' (static) Always false in case
	 * of a class
	 */
	public boolean isClassMember() {
		read_if_needed_();
		return _class_member;
	}

	/**
	 * Indicate if the member is 'volatile' Always false in case of a class
	 */
	public boolean isVolatile() {
		read_if_needed_();
		return _volatile;
	}

	/**
	 * returns the Java annotations
	 */
	public String javaAnnotations() {
		read_if_needed_();
		return _java_annotation;
	}

	@Override
	protected void read_cpp_() {
		super.read_cpp_();
		_cpp_visibility = aVisibility.fromInt(UmlCom.read_char());
	}

	@Override
	protected void read_java_() {
		super.read_java_();
		_java_annotation = UmlCom.read_string();
	}

	@Override
	protected void read_uml_() {
		super.read_uml_();
		_class_member = UmlCom.read_bool();
		_volatile = UmlCom.read_bool();
		_visibility = aVisibility.fromInt(UmlCom.read_char());
		_constraint = UmlCom.read_string();
	}

	/**
	 * set the constraint
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Constraint(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setConstraintCmd, v);
		UmlCom.check();
		_constraint = v;
	}

	/**
	 * Set the visibility for C++, DefaultVisibility is legal
	 * On error return FALSE in C++, produce a PlugoutException in Java, does
	 * not check that the class is (already) a typedef
	 */
	public void set_CppVisibility(aVisibility v) throws PlugoutException {
		if(v == aVisibility.DefaultVisibility) {
			throw new PlugoutException("Cannot be done");
		}
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setCppVisibilityCmd, (byte)v.value());
		UmlCom.check();
		_cpp_visibility = v;
	}

	/**
	 * Set if the member is a 'class member' (static), not significant for a
	 * class
	 * On error return FALSE in C++, produce a PlugoutException in Java, does
	 * not check that the class is (already) a typedef
	 */
	public void set_isClassMember(boolean y) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIsClassMemberCmd, y ? (byte)1 : (byte)0);
		UmlCom.check();
		_class_member = y;
	}

	/**
	 * Set if the member is 'volatile', not significant for a class
	 * On error return FALSE in C++, produce a PlugoutException in Java, does
	 * not check that the class is (already) a typedef
	 */
	public void set_isVolatile(boolean y) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIsVolatileCmd, y ? (byte)1 : (byte)0);
		UmlCom.check();
		_volatile = y;
	}

	/**
	 * set the Java annotations
	 * On error : return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_JavaAnnotations(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setJavaAnnotationCmd, v);
		UmlCom.check();
		_java_annotation = v;
	}

	/**
	 * Set the member visibility, DefaultVisibility is not legal
	 * On error return FALSE in C++, produce a PlugoutException in Java, does
	 * not check that the class is (already) a typedef
	 */
	public void set_Visibility(aVisibility v) throws PlugoutException {
		if(v == aVisibility.DefaultVisibility) {
			throw new PlugoutException("Cannot be done");
		}
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setVisibilityCmd, (byte)v.value());
		UmlCom.check();
		_visibility = v;
	}

	/**
	 * The member's visibility
	 */
	public aVisibility visibility() {
		read_if_needed_();
		return _visibility;
	}
}
