package org.bouml.plugout.apibase.umlitem;

import java.util.Hashtable;
import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.CmdFamily;
import org.bouml.plugout.apibase.com.MiscGlobalCmd;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;

/**
 * Base class of all the classes representing browser's objects.
 * In the API all for each 'system' class exists a 'user' class under your
 * responsability which inherits the 'system' class.
 * You do not have to change the 'system' classes, place your attributes and
 * operations in the corresponding 'user' classes : all the instances made by
 * the 'system' classes are instance of the corresponding 'user' classes.
 * You must never create or delete yourself a class of the API, use the Create
 * operation defined in the 'system' classes.
 * To not allow you to access to the 'system' attributes or to call 'system'
 * internal operations even by error, the 'system' internal parts are declared
 * 'private' rather than 'public' or 'protected'.
 */
public abstract class UmlItem {
	private static final Hashtable<Long, UmlItem> _all = new Hashtable<Long, UmlItem>(1000);

	/**
	 * The factory of the item in the browser of bouml.
	 */
	private static final UmlItemFactory umlItemFactory = UmlItemFactory.getSingleton();

	/**
	 * 'id' is an identifier returned by apply(), indicates if the tool is still
	 * running
	 */
	public static boolean isToolRunning(int id) {
		UmlCom.send_cmd(CmdFamily.miscGlobalCmd, MiscGlobalCmd._toolRunningCmd, id, "");
		return UmlCom.read_bool();
	}

	public static UmlItem[] markedItems() {
		UmlCom.send_cmd(CmdFamily.miscGlobalCmd, MiscGlobalCmd._allMarkedCmd);
		return UmlCom.read_item_list();
	}

	/**
	 * Internal, do NOT use it.
	 */
	public static UmlItem read_() {
		final long id = UmlCom.read_id();
		if(id == 0) {
			return null;
		}
		final int kind = UmlCom.read_char();
		final String name = UmlCom.read_string();
		final UmlItem result = _all.get(new Long(id));
		if(result == null) {
			return umlItemFactory.getInstance(kind, id, name);
		}
		return result;
	}

	private UmlItem[] _children;

	/**
	 * Do NOT use the next definitions
	 */
	private boolean _defined;

	private String _description;

	private Hashtable<String, String> _dict;

	private final long _identifier;

	private boolean _marked;

	protected int _modeler_id;

	private String _name;

	private UmlItem _parent;

	/**
	 * Note : protected in Java for internal reason, but do NOT access to this
	 * field yourself !
	 */
	protected String _stereotype;

	/**
	 * The constructor, do not call it yourself !
	 */
	public UmlItem(long id, String n) {
		_defined = false;
		_identifier = id;
		_name = n;
		_all.put(new Long(id), this);
	}

	/**
	 * Apply asynchronously the tool on the item, returns an identifier to call
	 * isToolDone().
	 */
	public int apply(String cmd) {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.applyCmd, cmd);
		return UmlCom.read_unsigned();
	}

	/**
	 * If the current stereotype is part of a profile add needed properties. In
	 * all cases remove extra properties whose keys contain two ':'.
	 * If the element is read-only, produce a PlugoutException.
	 */
	public void applyStereotype() throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.applyStereotypeCmd);
		UmlCom.check();
		unload(false, false);
	}

	/**
	 * Returns (in Java : a copy of) the children list, to go all over the
	 * browser tree.
	 */
	public UmlItem[] children() {
		if(_children == null) {
			read_children_();
		}
		return _children.clone();
	}

	/**
	 * return TRUE if the children exist and are visible (independently of the
	 * scroll bar) in the browser, else FALSE
	 */
	public boolean childrenVisible() {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.isOpenCmd);
		return UmlCom.read_bool();
	}

	/**
	 * internal, do NOT use it
	 */
	protected final UmlItem create_(anItemKind k, String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.createCmd, k, s);
		final UmlItem result = UmlItem.read_();
		if(result != null) {
			if(_children != null) {
				final int n = _children.length;
				final UmlItem[] ch = new UmlItem[n + 1];
				int index;
				for(index = 0 ; index != n ; index += 1) {
					ch[index] = _children[index];
				}
				ch[index] = result;
				_children = ch;
			}
			result._parent = this;
		} else {
			throw new PlugoutException("Can't be created.");
		}
		return result;
	}

	/**
	 * internal, do NOT use it
	 */
	public boolean defined_() {
		return _defined;
	}

	/**
	 * remove the element from the model, use it carefully because after that
	 * the element can't be used anymore by the plug-out
	 * On error : return FALSE in C++, produce a PlugoutException in Java
	 */
	public void deleteIt() throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.deleteCmd);
		UmlCom.check();
		parent().unload(true, false);
	}

	/**
	 * returns the description
	 */
	public String description() {
		read_if_needed_();
		return _description;
	}

	/**
	 * return a constant identifier, it is unique within a given kind of element
	 * (two classes can't have the same identifiers, but a class and a component
	 * may have the same identifier) except for the diagrams (a class diagram
	 * can't have the identifier of a component diagram)
	 */
	public int getIdentifier() {
		read_if_needed_();
		return _modeler_id;
	}

	/**
	 * Internal, do NOT use it.
	 */
	public final long identifier_() {
		return _identifier;
	}

	/**
	 * return TRUE in case the item is marked
	 */
	public boolean isMarked() {
		read_if_needed_();
		return _marked;
	}

	/**
	 * return TRUE in case the item may be modified, i.e. it is not an API base
	 * 'plug-out' class and the corresponding file(s) are not read-only.
	 */
	public boolean isWritable() {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.isWritableCmd);
		return UmlCom.read_bool();
	}

	/**
	 * returns the kind of the item
	 */
	public abstract anItemKind kind();

	/**
	 * If the parameter is null, move the current item to be the first child of
	 * its parent. Else the current item and the parameter must have the same
	 * parent, the current item is moved to be just after the parameter. On
	 * error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void moveAfter(UmlItem x) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.moveAfterCmd, x != null ? x.identifier_() : 0);
		UmlCom.check();
		parent().reread_children_if_needed_();
	}

	/**
	 * returns the name
	 */
	public String name() {
		return _name;
	}

	/**
	 * returns the parent (0/null if it is applied on the project itself), to go
	 * all over the browser tree
	 */
	public UmlItem parent() {
		if(_parent == null) {
			UmlCom.send_cmd(identifier_(), OnInstanceCmd.parentCmd);
			_parent = UmlItem.read_();
		}
		return _parent;
	}

	@SuppressWarnings("unchecked")
	public Hashtable<String, String> properties() {
		read_if_needed_();
		return _dict == null ? new Hashtable<String, String>() : (Hashtable<String, String>)_dict.clone();
	}

	/**
	 * In C++, in case the key specified by 'k' exist for the current object,
	 * the string 'v' is modified which the associated value and TRUE is
	 * returned, else returns FALSE.
	 * In Java return the value associated with the key specified by 'k' for the
	 * current object, null in case the property doesn't exist.
	 * These properties may be edited through the object dialog last tab named
	 * 'user'
	 */
	public String propertyValue(String k) {
		read_if_needed_();
		if(_dict == null) {
			return null;
		}
		return _dict.get(k);
	}

	/**
	 * Internal, do NOT use it.
	 */
	protected final void read_children_() {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.childrenCmd);
		_children = UmlCom.read_item_list();
		int n = _children.length;
		while(n-- != 0) {
			_children[n]._parent = this;
		}
	}

	/**
	 * Internal, do NOT use it.
	 */
	protected void read_cpp_() {
	}

	/**
	 * Internal, do NOT use it.
	 */
	protected void read_idl_() {
	}

	/**
	 * Internal, do NOT use it.
	 */
	protected final void read_if_needed_() {
		if(!_defined) {
			UmlCom.send_cmd(identifier_(), OnInstanceCmd.getDefCmd);
			read_uml_();
			read_cpp_();
			read_java_();
			read_php_();
			read_python_();
			read_idl_();
			_defined = true;
		}
	}

	/**
	 * Internal, do NOT use it.
	 */
	protected void read_java_() {
	}

	/**
	 * Internal, do NOT use it.
	 */
	protected void read_php_() {
	}

	/**
	 * Internal, do NOT use it.
	 */
	protected void read_python_() {
	}

	/**
	 * Internal, do NOT use it.
	 */
	protected void read_uml_() {
		_stereotype = UmlCom.read_string();
		int n = UmlCom.read_unsigned();
		if(n != 0) {
			_dict = new Hashtable<String, String>(n);
			String k;
			do {
				k = UmlCom.read_string();
				_dict.put(k, UmlCom.read_string());
			} while(--n != 0);
		} else {
			_dict = null;
		}
		_description = UmlCom.read_string();
		_marked = UmlCom.read_bool();
		_modeler_id = UmlCom.read_unsigned();
	}

	/**
	 * Returns the items referencing the current one. The result may contains
	 * UmlAttribute, UmlRelations, UmlNcRelations, UmlBaseOperation (their
	 * bodies are not taken into account) , UmlClass and UmlComponents.
	 */
	public UmlItem[] referencedBy() {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.childrenCmd);
		return UmlCom.read_item_list();
	}

	/**
	 * Internal, do NOT use it.
	 */
	protected final void reread_children_if_needed_() {
		if(_children != null) {
			read_children_();
		}
	}

	/**
	 * open the item in the browser (its children are visible) if y is TRUE,
	 * else close it (its children are not visible). May be applied on items
	 * without children. The item is also selected in the browser, so
	 * set_ChildrenVisible(childrenVisible()) is the right way to select the
	 * item in the browser.
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_childrenVisible(boolean y) throws PlugoutException {
		UmlCom.send_cmd(_identifier, OnInstanceCmd.setOpenCmd, y ? (byte)1 : (byte)0);
		UmlCom.check();
	}

	/**
	 * to set the description On error return FALSE in C++, produce a
	 * PlugoutException in Java
	 */
	public void set_Description(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setDescriptionCmd, s);
		UmlCom.check();
		if(_defined) {
			_description = s;
		}
	}

	/**
	 * to mark/unmark the current item. The project cannot marked On error
	 * return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isMarked(boolean y) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setMarkedCmd, y ? (byte)1 : (byte)0);
		UmlCom.check();
		_marked = y;
	}

	/**
	 * to set the name
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Name(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setNameCmd, s);
		UmlCom.check();
		_name = s;
	}

	/**
	 * to set (may be insert a new) the value 'v' associated to the key 'k'
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_PropertyValue(String k, String v) throws PlugoutException {
		read_if_needed_();
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setCoupleValueCmd, k, v);
		UmlCom.check();
		if(_dict == null) {
			_dict = new Hashtable<String, String>();
		}
		_dict.put(k, v);
	}

	/**
	 * to set the stereotype On error return FALSE in C++, produce a
	 * PlugoutException in Java
	 */
	public void set_Stereotype(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setStereotypeCmd, s);
		UmlCom.check();
		if(_defined) {
			_stereotype = s;
		}
	}

	/**
	 * returns the stereotype
	 */
	public String stereotype() {
		read_if_needed_();
		return _stereotype;
	}

	/**
	 * Returns the absolute path name of the file memoring the Item. In general
	 * an item is saved in a file associated to its package, the exceptions are
	 * :
	 * - the body of the operations of a class which definition contains the
	 * keyword '{$body}'. This path name is returned when you apply supportFile
	 * on any operation of the class.
	 * - the drawing of the diagrams, this path name is returned when you apply
	 * supportFile on the diagram.
	 * - the configuration of the tools edited throw the entry 'Tools Settings'
	 * of the menu Tools is saved in the file 'tools'
	 * - the configuration of the '#include' and 'using' forms associated to the
	 * external type and edited through the last C++ tab of the 'Generation
	 * Settings' dialog is saved in the file 'cpp_includes'
	 * - the configuration of the 'imports' forms associated to the external
	 * types and edited through the last Java tab of the 'Generation Settings'
	 * dialog is saved in the file 'java_imports'
	 * - the configuration of the '#include' forms associated to the external
	 * type and edited through the last Idl tab of the 'Generation Settings'
	 * dialog is saved in the file 'idl_includes'
	 */
	public String supportFile() {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.supportFileCmd);
		return UmlCom.read_string();
	}

	/**
	 * To unload the object to free memory, it will be reloaded automatically if
	 * needed. Recursively done for the sub items if 'rec' is TRUE.
	 * If 'del' is true the sub items are deleted in C++, and removed from the
	 * internal dictionnary in C++ and Java (to allow it to be garbaged), you
	 * will have to call Children() to re-access to them.
	 */
	public void unload(boolean rec, boolean del) {
		_defined = false;
		_stereotype = null;
		_dict = null;
		_description = null;
		if(_children != null) {
			if(rec) {
				for(int chindex = 0 ; chindex != _children.length ; chindex += 1) {
					_children[chindex].unload(true, del);
					if(del) {
						_all.remove(new Long(_children[chindex].identifier_()));
					}
				}
			}
			_children = null;
		}
	}
}
