package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.aRelationKind;
import org.bouml.plugout.apibase.type.anItemKind;

/**
 * Manage the relations between classes
 */
public class UmlRelation extends UmlClassMember {
	/**
	 * returns a new relation of the given 'kind' from 'start' to 'end'
	 * In case it cannot be created (the name is already used or invalid,
	 * 'parent' cannot contain it etc ...) return 0 in C++ and produce a
	 * PlugoutException in Java
	 */
	public static UmlRelation create(aRelationKind kind, UmlClass start, UmlClass end) throws PlugoutException {
		UmlCom.send_cmd(start.identifier_(), OnInstanceCmd.createCmd, anItemKind.aRelation, kind, end.identifier_());
		final UmlRelation result = (UmlRelation)UmlItem.read_();
		if(result != null) {
			start.reread_children_if_needed_();
			switch(kind.value()) {
				case aRelationKind._anAssociation:
				case aRelationKind._anAggregation:
				case aRelationKind._anAggregationByValue:
					// bi-directional
					end.reread_children_if_needed_();
					break;
				case aRelationKind._aGeneralisation:
				case aRelationKind._aRealization:
					if(end.formals().length != 0) {
						// to update actuals & base_type
						start.reread_if_needed_();
					}
				default:
					break;
			}
		} else {
			throw new PlugoutException("can't be created");
		}
		return result;
	}

	private UmlTypeSpec _association;

	private boolean _cpp_mutable;

	private boolean _cpp_virtual_inheritance;

	private String _default_value;

	private boolean _derived;

	private boolean _derived_union;

	private UmlOperation _get_oper;

	/**
	 * exclusive with idl_explicit_case
	 */
	private UmlAttribute _idl_case;

	private String _idl_explicit_case;

	private boolean _idl_truncatable;

	private boolean _java_transient;

	private String _multiplicity;

	private boolean _ordered;

	private boolean _read_only;

	private aRelationKind _rel_kind;

	private String _role_name;

	private UmlClass _role_type;

	private UmlOperation _set_oper;

	private boolean _unique;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlRelation(long id, String n) {
		super(id, n);
	}

	/**
	 * to generate an associated 'get' operation
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void addGetOperation() throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.addGetOperCmd);
		UmlCom.check();
		reread_children_if_needed_();
	}

	/**
	 * to generate an associated 'set' operation
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void addSetOperation() throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.addSetOperCmd);
		UmlCom.check();
		reread_children_if_needed_();
	}

	/**
	 * return the associated class/type, may be an empty spec
	 */
	public UmlTypeSpec association() {
		read_if_needed_();
		return _association;
	}

	/**
	 * indicates if the inheritance is virtual in C++, returns TRUE if yes
	 */
	public boolean cppVirtualInheritance() {
		read_if_needed_();
		return _cpp_virtual_inheritance;
	}

	/**
	 * returns the default relation value, may be an empty string
	 */
	public String defaultValue() {
		read_if_needed_();
		return _default_value;
	}

	/**
	 * returns the 'get' operation of the relation, or 0 if it does not exist
	 */
	public UmlOperation getOperation() {
		read_if_needed_();
		return _get_oper;
	}

	/**
	 * in case the relation is an IDL union's member returns the corresponding
	 * 'case', an empty string in case it is not specified
	 */
	public String idlCase() {
		read_if_needed_();
		return _idl_case != null ? _idl_case.name() : _idl_explicit_case;
	}

	/**
	 * Indicate if the relation is 'mutable'
	 */
	public boolean isCppMutable() {
		read_if_needed_();
		return _cpp_mutable;
	}

	/**
	 * return the property 'derived'
	 */
	public boolean isDerived() {
		read_if_needed_();
		return _derived;
	}

	/**
	 * return the property 'derived union'
	 */
	public boolean isDerivedUnion() {
		read_if_needed_();
		return _derived_union;
	}

	/**
	 * indicates if the relation is 'truncatable', returns TRUE if yes have
	 * sense only for a valuetype inheritance
	 */
	public boolean isIdlTruncatableInheritance() {
		read_if_needed_();
		return _idl_truncatable;
	}

	/**
	 * indicates if the relation is 'transient', returns TRUE if yes
	 */
	public boolean isJavaTransient() {
		read_if_needed_();
		return _java_transient;
	}

	/**
	 * return the property 'ordered'
	 */
	public boolean isOrdered() {
		read_if_needed_();
		return _ordered;
	}

	/**
	 * indicates if the relation is read only, returns TRUE if yes
	 */
	public boolean isReadOnly() {
		read_if_needed_();
		return _read_only;
	}

	/**
	 * return the property 'unique'
	 */
	public boolean isUnique() {
		read_if_needed_();
		return _unique;
	}

	/**
	 * returns the kind of the item
	 */
	@Override
	public anItemKind kind() {
		return anItemKind.aRelation;
	}

	/**
	 * returns the multiplicity (may be an empty string)
	 */
	public String multiplicity() {
		read_if_needed_();
		return _multiplicity;
	}

	@Override
	protected void read_cpp_() {
		super.read_cpp_();
		_cpp_mutable = UmlCom.read_bool();
		_cpp_virtual_inheritance = UmlCom.read_bool();
	}

	@Override
	protected void read_idl_() {
		super.read_idl_();
		_idl_truncatable = UmlCom.read_bool();
		_idl_case = (UmlAttribute)UmlItem.read_();
		if(_idl_case == null) {
			_idl_explicit_case = UmlCom.read_string();
		}
	}

	@Override
	protected void read_java_() {
		super.read_java_();
		_java_transient = UmlCom.read_bool();
	}

	@Override
	protected void read_php_() {
		super.read_php_();
	}

	@Override
	protected void read_python_() {
		super.read_python_();
	}

	@Override
	protected void read_uml_() {
		super.read_uml_();
		_rel_kind = aRelationKind.fromInt(UmlCom.read_char());
		_role_type = (UmlClass)UmlItem.read_();
		_association = new UmlTypeSpec();
		_association.type = (UmlClass)UmlItem.read_();
		if(_association.type == null) {
			_association.explicit_type = UmlCom.read_string();
		}
		_role_name = UmlCom.read_string();
		_multiplicity = UmlCom.read_string();
		_default_value = UmlCom.read_string();
		_read_only = UmlCom.read_bool();
		_derived = UmlCom.read_bool();
		_derived_union = UmlCom.read_bool();
		_ordered = UmlCom.read_bool();
		_unique = UmlCom.read_bool();
		_get_oper = (UmlOperation)UmlItem.read_();
		_set_oper = (UmlOperation)UmlItem.read_();
	}

	/**
	 * returns the kind of the relation
	 */
	public aRelationKind relationKind() {
		read_if_needed_();
		return _rel_kind;
	}

	/**
	 * returns the name of the role
	 */
	public String roleName() {
		read_if_needed_();
		return _role_name;
	}

	/**
	 * returns the 'end' class (the 'start' class is the parent of the relation)
	 * no set !
	 */
	public UmlClass roleType() {
		read_if_needed_();
		return _role_type;
	}

	/**
	 * set the associated class/type, may be an empty spec
	 * On error : return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Association(UmlTypeSpec t) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setRelationAssocClassCmd, t);
		UmlCom.check();
		_association = t;
		final UmlRelation other = (UmlRelation)UmlItem.read_();
		if(other != null) {
			other._association = t;
		}
	}

	/**
	 * to set if the inheritance is virtual in C++
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_CppVirtualInheritance(boolean y) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setCppVirtualInheritanceCmd, y ? (byte)1 : (byte)0);
		UmlCom.check();
		_cpp_virtual_inheritance = y;
	}

	/**
	 * to set the default relation value ("" allowed)
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_DefaultValue(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setDefaultValueCmd, s);
		UmlCom.check();
		_default_value = s;
	}

	/**
	 * to set the 'case' even the relation is not (already) known as an IDL
	 * union's member
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_IdlCase(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIdlCaseCmd, 0, s);
		UmlCom.check();
		_idl_case = null;
		_idl_explicit_case = s;
	}

	/**
	 * to set the 'case' even the relation is not (already) known as an IDL
	 * union's member
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_IdlCase(UmlAttribute a) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIdlCaseCmd, a.identifier_(), "");
		UmlCom.check();
		_idl_case = a;
		_idl_explicit_case = null;
	}

	/**
	 * Set if the relation is 'mutable'
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isCppMutable(boolean y) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIsCppMutableCmd, y ? (byte)1 : (byte)0);
		UmlCom.check();
		_cpp_mutable = y;
	}

	/**
	 * Set the properties 'derived' and 'union'
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isDerived(boolean is_derived, boolean is_union) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setDerivedCmd, (byte)((is_derived ? 1 : 0) + (is_union ? 2 : 0)));
		UmlCom.check();
		_derived = is_derived;
		_derived_union = is_union;
	}

	/**
	 * to set if the inheritance is 'truncatable'
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isIdlTruncatableInheritance(boolean y) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIsIdlTruncatableCmd, y ? (byte)1 : (byte)0);
		UmlCom.check();
		_idl_truncatable = y;
	}

	/**
	 * to set the 'transient' state of the relation
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isJavaTransient(boolean y) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIsJavaTransientCmd, y ? (byte)1 : (byte)0);
		UmlCom.check();
		_java_transient = y;
	}

	/**
	 * set the property 'ordered'
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isOrdered(boolean v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setOrderingCmd, v ? 1 : 0);
		UmlCom.check();
		_ordered = v;
	}

	/**
	 * to set the 'read only' state of the relation
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isReadOnly(boolean y) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIsReadOnlyCmd, y ? (byte)1 : (byte)0);
		UmlCom.check();
		_read_only = y;
	}

	/**
	 * set the property 'unique'
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isUnique(boolean v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setUniqueCmd, v ? 1 : 0);
		UmlCom.check();
		_unique = v;
	}

	/**
	 * to set the multiplicity
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Multiplicity(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setMultiplicityCmd, s);
		UmlCom.check();
		_multiplicity = s;
	}

	/**
	 * to set the name of the role
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_RoleName(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setRoleNameCmd, s);
		UmlCom.check();
		_role_name = s;
	}

	/**
	 * to set the stereotype
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 * redefined in case the relation is bidirectional to set the stereotype of
	 * the relation corresponding to the other direction
	 */
	@Override
	public void set_Stereotype(String s) throws PlugoutException {
		super.set_Stereotype(s);
		final UmlRelation other = (UmlRelation)UmlItem.read_();
		if(other != null) {
			other._stereotype = s;
		}
	}

	/**
	 * returns the 'set' operation of the relation, or 0 if it does not exist
	 */
	public UmlOperation setOperation() {
		read_if_needed_();
		return _set_oper;
	}

	/**
	 * if 'first' is true returns the relation associated to the first role,
	 * else the relation associated to the second role or 0/null if the relation
	 * is uni directional
	 */
	public UmlRelation side(boolean first) {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.sideCmd, first ? (byte)1 : (byte)0);
		return (UmlRelation)UmlItem.read_();
	}

	@Override
	public void unload(boolean rec, boolean del) {
		_role_name = null;
		_multiplicity = null;
		_default_value = null;
		_idl_explicit_case = null;
		super.unload(rec, del);
	}
}
