package org.bouml.plugout.sort.model;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.aRelationKind;
import org.bouml.plugout.apibase.type.anItemKind;
import org.bouml.plugout.apibase.umlitem.UmlItem;
import org.bouml.plugout.sort.resource.ISortConstant;

/**
 * Comparator for UML item. It can sort the children of an UML item.
 */
public final class SortItemComparator implements Comparator<ISortItem> {
	private static SortItemComparator singleton;

	private int nbSortingItems;

	public static SortItemComparator getSingleton() {
		return singleton != null ? singleton : (singleton = new SortItemComparator());
	}

	private SortItemComparator() {
		nbSortingItems = 0;
	}

	public int compare(ISortItem item1, ISortItem item2) {
		if(item1.kind().value() == anItemKind._aRelation && item2.kind().value() == anItemKind._aRelation) {
			final UmlRelation relation1 = (UmlRelation)item1;
			final UmlRelation relation2 = (UmlRelation)item2;
			final int relation1Kind = relation1.relationKind().value();
			if(relation1Kind == relation2.relationKind().value() && relation1Kind == aRelationKind._aDependency) {
				final String stereotype1 = item1.stereotype();
				final String stereotype2 = item2.stereotype();
				if(stereotype1.equals(stereotype2)) {
					if(ISortConstant.STEREOTYPE_IMPORT.equals(stereotype1)) {
						return relation1.getJavaImport().compareTo(relation2.getJavaImport());
					} else if(ISortConstant.STEREOTYPE_INCLUDE.equals(stereotype1)) {
						return relation1.getCppHDir().compareTo(relation2.getCppHDir());
					}
				} else {
					return stereotype1.compareTo(stereotype2);
				}
			}
		}
		final int w1 = item1.getOrderWeight(), w2 = item2.getOrderWeight();
		if(w1 != w2) {
			return w1 - w2;
		} else {
			final String stereotype1 = item1.stereotype();
			final String stereotype2 = item2.stereotype();
			return stereotype1.equals(stereotype2) ? item1.name().compareTo(item2.name()) : stereotype1.compareTo(stereotype2);
		}
	}

	public int getNbSortingItems() {
		return nbSortingItems;
	}

	public void sort(UmlItem[] childrens) {
		final List<ISortItem> sortItems = new ArrayList<ISortItem>(childrens.length);
		for(final UmlItem children : childrens) {
			if(children instanceof ISortItem) {
				sortItems.add((ISortItem)children);
			}
		}
		if(!sortItems.isEmpty()) {
			UmlCom.trace("Sorting : " + ((UmlItem)sortItems.get(0)).parent().name());
			nbSortingItems++;
			// Sort the content of the item.
			Collections.sort(sortItems, singleton);
			ISortItem previous = null;
			for(final ISortItem sortItem : sortItems) {
				sortItem.moveAfter((UmlItem)previous);
				previous = sortItem;
			}
			// Sort the child of the item.
			for(final ISortItem sortItem : sortItems) {
				sortItem.sort();
			}
		}
	}

}
