package org.bouml.plugout.sort.controller;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.sort.model.SortItemKind;
import org.bouml.plugout.sort.model.SortManager;
import org.bouml.plugout.sort.util.ImageUtility;
import org.bouml.plugout.sort.view.SortItemComponent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;

/**
 * Controller of the component SortItemComponent. It fill the table with the image and position of
 * item.
 */
public class SortItemController {
  public interface ISortItemControllerListener {
  	void configurationChangedEvent(boolean modified) ;

  }

	private static final SortItemKind[] allItems = SortItemKind.values();

	private final Comparator<SortItemKind> comparator;

	private final SortItemComponent component;

	private final List<SortItemKind> items;

	private final ISortItemControllerListener listener;

	public SortItemController(SortItemComponent component, ISortItemControllerListener listener, SortItemKind...itemKinds ) {
		this(component, listener);
		for(final SortItemKind itemKind : itemKinds) {
			items.add(itemKind);
		}
	}

	public SortItemController(SortItemComponent component, ISortItemControllerListener listener, String prefix) {
		this(component, listener);
		for(final SortItemKind itemKind : allItems) {
			if(prefix.equals(itemKind.getPrefix())) {
				items.add(itemKind);
			}
		}
	}

	private SortItemController(SortItemComponent component, ISortItemControllerListener listener) {
		this.component = component;
		comparator = new Comparator<SortItemKind>() {
			public int compare(SortItemKind o1, SortItemKind o2) {
				final int position1 = o1.getPosition();
				final int position2 = o2.getPosition();
				if(position1 > position2) {
					return 1;
				} else if(position1 < position2) {
					return -1;
				} else {
					return o1.getLabel().compareTo(o2.getLabel());
				}
			}
		};
		items = new ArrayList<SortItemKind>();
		this.listener = listener;
	}

	public void defineListener() {
		component.getButtonDown().addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				managedButtonDownEvent();
			}
		});
		component.getButtonUp().addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				managedButtonUpEvent();
			}
		});
	}

	public SortItemComponent getComponent() {
		return component;
	}

	public void loadData() {
		final int itemsSize = items.size();
		final Table table = component.getTable();
		for(int index = 0 ; index < itemsSize ; index++) {
			new TableItem(table, SWT.NONE).getBounds(0);
		}
		table.select(0);
		updateTable();
	}

	private final int getMaxPosition() {
		int maxPosition = 0;
		for(final SortItemKind item : items) {
			maxPosition = Math.max(item.getPosition(), maxPosition);
		}
		return maxPosition;
	}

	private final void managedButtonDownEvent() {
		final int selectionIndex = component.getTable().getSelectionIndex();
		if(selectionIndex != -1) {
			final SortItemKind itemToSort = items.get(selectionIndex);
			final int positionToSort = itemToSort.getPosition();
			int tmpPosition;
			if(oneItemAtPosition(itemToSort)) {
				if(positionToSort != getMaxPosition()) {
					for(final SortItemKind item : items) {
						tmpPosition = item.getPosition();
						if(tmpPosition > positionToSort) {
							item.setPosition(tmpPosition - 1);
						}
					}
				}
			} else {
				itemToSort.setPosition(positionToSort + 1);
				for(final SortItemKind item : items) {
					tmpPosition = item.getPosition();
					if(tmpPosition > positionToSort && itemToSort != item) {
						item.setPosition(tmpPosition + 1);
					}
				}
			}
			updateTable();
			selectItem(itemToSort);
			listener.configurationChangedEvent(SortManager.getSingleton().isPropertiesChanged());
		}
	}

	private final void managedButtonUpEvent() {
		final int selectionIndex = component.getTable().getSelectionIndex();
		if(selectionIndex != -1) {
			final SortItemKind itemToSort = items.get(selectionIndex);
			final int positionToSort = itemToSort.getPosition();
			int tmpPosition;
			if(oneItemAtPosition(itemToSort)) {
				if(positionToSort != 1) {
					itemToSort.setPosition(positionToSort - 1);
					for(final SortItemKind item : items) {
						tmpPosition = item.getPosition();
						if(tmpPosition > positionToSort) {
							item.setPosition(tmpPosition - 1);
						}
					}
				}
			} else {
				for(final SortItemKind item : items) {
					tmpPosition = item.getPosition();
					if(tmpPosition >= positionToSort && itemToSort != item) {
						item.setPosition(tmpPosition + 1);
					}
				}
			}
			updateTable();
			selectItem(itemToSort);
			listener.configurationChangedEvent(SortManager.getSingleton().isPropertiesChanged());
		}
	}

	private final boolean oneItemAtPosition(SortItemKind item) {
		final int position = item.getPosition();
		for(final SortItemKind tmpItem : items) {
			if(tmpItem != item && tmpItem.getPosition() == position) {
				return false;
			}
		}
		return true;
	}

	private final void selectItem(SortItemKind itemToSort) {
		int index = 0;
		for(final SortItemKind item : items) {
			if(item == itemToSort) {
				component.getTable().select(index);
				return;
			} else {
				index++;
			}
		}
		// This case is not possible.
		throw new PlugoutException("Unable to make a selection.");
	}

	private final void updateTable() {
		Collections.sort(items, comparator);
		final Table table = component.getTable();
		final int itemCount = table.getItemCount();
		TableItem tmpTableItem;
		SortItemKind tmpItem;
		for(int index = 0 ; index < itemCount ; index++) {
			tmpTableItem = table.getItem(index);
			tmpItem = items.get(index);
			tmpTableItem.setImage(ImageUtility.loadImage(tmpItem.getImagePath()));
			tmpTableItem.setText(" - " + tmpItem.getPosition() + " - " + tmpItem.getLabel());
		}
	}

}
