package org.bouml.plugout.sort.model;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.Properties;
import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.type.aRelationKind;
import org.bouml.plugout.apibase.type.aVisibility;
import org.bouml.plugout.apibase.type.anItemKind;
import org.bouml.plugout.apibase.umlitem.UmlClassMember;
import org.bouml.plugout.sort.resource.ISortConstant;

/**
 * Class of management for the configuration file sort.properties of plug-out sort.
 */
public final class SortManager {
	private static SortManager singleton = null;

	private final Properties newProperties;

	private final Properties oldProperties;

	public static SortManager getSingleton() {
		return singleton != null ? singleton : (singleton = new SortManager());
	}

	private SortManager() {
		try {
			newProperties = new Properties();
			oldProperties = new Properties();
			
			final InputStream stream = ClassLoader.getSystemResourceAsStream(ISortConstant.SORT_PROPERTIES);
			if(stream == null) {
				throw new PlugoutException("The file sort.properties was not found in binary repertory of plugout sort.");
			}
			newProperties.load(stream);
			for(final Object key : newProperties.keySet()) {
				oldProperties.put(key, newProperties.getProperty((String)key));
			}
			stream.close();
			checkSortProperties();
		} catch(final Exception exception) {
			throw new PlugoutException(exception.getMessage());
		}
	}

	public int getClassMemberOrderWeight(UmlClassMember classMember) {
		final int value = classMember.kind().value();
		final int visibility = getVisibilityOrderWeight(classMember.visibility());
		if(value == anItemKind._aRelation) {
			final int relationKindValue = ((UmlRelation)classMember).relationKind().value();
			if(relationKindValue == aRelationKind._aDependency || relationKindValue == aRelationKind._aGeneralisation || relationKindValue == aRelationKind._aRealization) {
				return 1000 * getOrderWeight(SortItemKind.Class_Relation) + 10 * getRelationOrderWeight(((UmlRelation)classMember).relationKind()) + visibility;
			} else {
				return 1000 * getOrderWeight(SortItemKind.Class_Attribute) + 100 * getStaticMemberOrderWeight(classMember) + 10 * getAttributeOrderWeight(classMember) + visibility;
			}
		} else if(value == anItemKind._aClass) {
			final String stereotype = classMember.stereotype();
			final int orderWeight;
			if(ISortConstant.STEREOTYPE_ACTOR.equals(stereotype)) {
				orderWeight = getOrderWeight(SortItemKind.Class_Actor);
			} else if(ISortConstant.STEREOTYPE_STEREOTYPE.equals(stereotype)) {
				orderWeight = getOrderWeight(SortItemKind.Class_Stereotype);
			} else {
				orderWeight = getOrderWeight(SortItemKind.Class_Class);
			}
			return 1000 * orderWeight + visibility;
		} else if(value == anItemKind._anAttribute) {
			return 1000 * getOrderWeight(SortItemKind.Class_Attribute) + 100 * getStaticMemberOrderWeight(classMember) + 10 * getAttributeOrderWeight(classMember) + visibility;
		} else if(value == anItemKind._anOperation) {
			return 1000 * getOrderWeight(SortItemKind.Class_Operation) + 100 * getStaticMemberOrderWeight(classMember) + 10 * getOperationOrderWeight(((UmlOperation)classMember).isConstructor()) + visibility;
		}
		throw new PlugoutException("The class member is unknow.");
	}

	public int getOrderWeight(SortItemKind item) {
		return Integer.valueOf(newProperties.getProperty(item.name()));
	}

	public String getValueOfKey(SortItemKind item) {
		return newProperties.getProperty(item.name());
	}

	public boolean isPropertiesChanged() {
		return !oldProperties.equals(newProperties);
	}

	public void saveProperties() {
		try {
			final FileOutputStream fileOutputStream = new FileOutputStream(new File(ClassLoader.getSystemResource(ISortConstant.SORT_PROPERTIES).toURI()));
			newProperties.store(fileOutputStream, "Configuration file to sort out the UML-items of a class.");
			fileOutputStream.close();
		} catch(final Exception exception) {
			throw new PlugoutException("Impossible to save the configuration : " + exception.getMessage());
		}
	}

	public void setValueOfKey(SortItemKind item, String value) {
		newProperties.put(item.name(), value);
	}

	private final void checkSortProperties() throws Exception {
		final StringBuilder errors = new StringBuilder();
		for(final SortItemKind item : SortItemKind.values()) {
			if(!newProperties.containsKey(item.name())) {
				errors.append("Property '" + item.name() + "' was not found in sort.propertites.<br>");
			}
		}
		if(errors.length() > 0) {
			throw new PlugoutException(errors.toString());
		}
	}

	private final int getAttributeOrderWeight(UmlClassMember classMember) {
		if(classMember instanceof UmlAttribute) {
			return getOrderWeight(SortItemKind.Class_BasicAttribute);
		} else if(classMember instanceof UmlRelation) {
			final int value = ((UmlRelation)classMember).relationKind().value();
			if(value == aRelationKind._anAssociation) {
				return getOrderWeight(SortItemKind.Class_Association);
			} else if(value == aRelationKind._aDirectionalAssociation) {
				return getOrderWeight(SortItemKind.Class_DirectionalAssociation);
			} else if(value == aRelationKind._anAggregation) {
				return getOrderWeight(SortItemKind.Class_Aggregation);
			} else if(value == aRelationKind._anAggregationByValue) {
				return getOrderWeight(SortItemKind.Class_AggregationByValue);
			} else if(value == aRelationKind._aDirectionalAggregation) {
				return getOrderWeight(SortItemKind.Class_DirectionalAggregation);
			} else if(value == aRelationKind._aDirectionalAggregationByValue) {
				return getOrderWeight(SortItemKind.Class_DirectionalAggregationByValue);
			}
			throw new PlugoutException("The parameter 'classMember' is not an attribute relation.");
		}
		throw new PlugoutException("The parameter 'classMember' must be an instance of UmlAttribute or UmlRelation.");
	}

	private final int getOperationOrderWeight(boolean isConstructor) {
		if("true".equals(newProperties.getProperty("Class_Constructor"))) {
			return isConstructor ? 0 : 1;
		} else {
			return 0;
		}
	}

	private final int getRelationOrderWeight(aRelationKind relationKind) {
		final int value = relationKind.value();
		if(value == aRelationKind._aDependency) {
			return getOrderWeight(SortItemKind.Class_Dependency);
		} else if(value == aRelationKind._aRealization) {
			return getOrderWeight(SortItemKind.Class_Realization);
		} else if(value == aRelationKind._aGeneralisation) {
			return getOrderWeight(SortItemKind.Class_Generalisation);
		}
		throw new PlugoutException("The parameter 'relation' is wrong for a relation.");
	}

	private final int getStaticMemberOrderWeight(UmlClassMember classMember) {
		if(classMember.isClassMember()) {
			return "true".equals(newProperties.getProperty("Class_StaticMember")) ? 1 : 2;
		} else {
			return 2;
		}
	}

	private final int getVisibilityOrderWeight(aVisibility visibility) {
		final int value = visibility.value();
		if(value == aVisibility._PublicVisibility) {
			return getOrderWeight(SortItemKind.Class_VisibilityPublic);
		} else if(value == aVisibility._ProtectedVisibility) {
			return getOrderWeight(SortItemKind.Class_VisibilityProtected);
		} else if(value == aVisibility._PrivateVisibility) {
			return getOrderWeight(SortItemKind.Class_VisibilityPrivate);
		} else if(value == aVisibility._PackageVisibility) {
			return getOrderWeight(SortItemKind.Class_VisibilityPackage);
		} else {
			return getOrderWeight(SortItemKind.Class_VisibilityDefault);
		}
	}

}
